
/**
 * Title:        ASCII File Resource Adapter<p>
 * Description:  This project is about using existing Java
 * application level classes and encapsulating them
 * in a JCA 1.0 compliant resource adapter<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iConexio.jca.asciiFileRA;

import javax.resource.*;
import javax.resource.spi.*;
import javax.resource.spi.security.PasswordCredential;
import javax.resource.spi.IllegalStateException;
import javax.resource.spi.SecurityException;
import javax.resource.NotSupportedException;
import java.io.*;
import java.util.*;
import javax.security.auth.Subject;

public class ASCIIFileManagedConnection implements ManagedConnection

{

    ASCIIFileManagedConnection(ManagedConnectionFactory mcf,
                         PasswordCredential passCred)
    {
        this.mcf = mcf;
        theCredentials = passCred;
        this.asciiFileName = null;
        this.metadataFileName = null;
        connectionSet = new HashSet();
    }

    public void setManagedConnectionFactory(ManagedConnectionFactory mcf)
    {
        this.mcf = mcf;
    }

    public ManagedConnectionFactory getManagedConnectionFactory()
    {
        return this.mcf;
    }

    // This method is called by the application server on behalf of the
    // application component. The application component passes information
    // to the application server via connectionRequestInfo instance.
    public Object getConnection(Subject subject,
                                ConnectionRequestInfo cRequestInfo)
        throws ResourceException
    {
      // Retrieve the password credentials passed in the ConnectionRegInfo
        PasswordCredential pc =
            Util.getPasswordCredential(mcf, subject, cRequestInfo);
        if (!Util.isPasswordCredentialEqual(pc, theCredentials)) {
            throw new SecurityException("Principal does not match." +
                                        "Reauthentication not supported");
        }

        AFConnectionRequestInfo afcri = (AFConnectionRequestInfo)cRequestInfo;
        if (this.asciiFileName == null)
        {
          this.asciiFileName = new String(afcri.getASCIIFileName());
          this.metadataFileName = new String(afcri.getMetadataFileName());
        }
        if (afcri.getASCIIFileName().equalsIgnoreCase(this.asciiFileName)
                   == false)
        {
           // This managed connection is not managing the file requested by the
           // application component in the connection request info
           throw new ResourceException("ASCII file name and managed connection"
                                       + " do not match");
        }
        // Ensure that this managed connection is still valid.
        checkIfDestroyed();
        ASCIIFileConnection aCon = new ASCIIFileConnection(this);
        addCciConnection(aCon); // Add the new connection to the set
        return aCon;
    }

    public void destroy() throws ResourceException
    {
      if (destroyed) return;
      destroyed = true;
      Iterator it = connectionSet.iterator();
      while (it.hasNext())
      {
        ASCIIFileConnection afCon = (ASCIIFileConnection) it.next();
        afCon.invalidate();
      }
      connectionSet.clear();
    }

    public void cleanup() throws ResourceException
    {
      checkIfDestroyed();
      Iterator it = connectionSet.iterator();
      while (it.hasNext())
      {
        ASCIIFileConnection afCon = (ASCIIFileConnection) it.next();
        afCon.invalidate();
      }
      connectionSet.clear();
    }

    public void associateConnection(Object theCon)
        throws ResourceException
    {
      checkIfDestroyed();
      if (theCon instanceof ASCIIFileConnection)
      {
            ASCIIFileConnection afCon = (ASCIIFIleConnection) theCon;
            afCon.associateConnection(this);
      }
      else
      {
        throw new IllegalStateException("Invalid connection object: " +
                                     theCon);
      }
    }

    public ManagedConnectionMetaData getMetaData() throws ResourceException
    {
        checkIfDestroyed();
        return new AFManagedConnectionMetaData(this);
    }

    public void setLogWriter(PrintWriter out) throws ResourceException
    {
        this.logWriter = out;
    }

    public PrintWriter getLogWriter() throws ResourceException
    {
        return logWriter;
    }

    boolean isDestroyed()
    {
        return destroyed;
    }

    PasswordCredential getPasswordCredential()
    {
        return passCred;
    }

    public void removeASCIIFileConnection(ASCIIFileConnection afCon)
    {
        connectionSet.remove(afCon);
    }

    public void addASCIIFileConnection(CciConnection afCon)
    {
        connectionSet.add(afCon);
    }

    private void checkIfDestroyed() throws ResourceException
    {
        if (destroyed)
        {
            throw new IllegalStateException("Managed connection is closed");
        }
    }
  public String getASCIIFileName()
  {
    return(asciiFileName);
  }

  // The managed connection instance keeps track of the data file name it
  // manages as well as the meta data file name associated with it.
  // The example adapter assumes that these files are accessible on the local
  // machine. If these files exist on a remote machine then a different set
  // of functions may be necessary to access the files over the network.
  private String asciiFileName;
  private String metadataFileName;

  // These variables hold the managed connection factory used to create this
  // instance and the a container (set) to hold application level connections
  // or handles to the adapter.
  private ManagedConnectionFactory mcf;
  private Set connectionSet;

  // This adapter supports the basic password credentials. These credentials
  // have no bearing on the ASCII data file and its permissions although it
  // can be programmed to check the ascii files ownership in a UNIX file system.
  private PasswordCredential theCredentials;

  private PrintWriter logWriter;
  private boolean connectionDestroyed; // Flag indicating that the connection
                                       // is destroyed.
}

